function workerize(code) {
    var exports = {};
    var exportsObjName = "__EXPORTS_" + (Math.random().toString().substring(2)) + "__";
    if (typeof code === 'function') 
        { code = "(" + (toCode(code)) + ")(" + exportsObjName + ")"; }
    code = toCjs(code, exportsObjName, exports);
    code += "\n(" + (toCode(setup)) + ")(self, " + exportsObjName + ", {})";
    var blob = new Blob([code], {
        type: 'application/javascript'
    }), url = URL.createObjectURL(blob), worker = new Worker(url), counter = 0, callbacks = {};
    worker.kill = (function (signal) {
        worker.postMessage({
            type: 'KILL',
            signal: signal
        });
        setTimeout(worker.terminate);
    });
    worker.terminate = (function () {
        URL.revokeObjectURL(url);
        worker.terminate();
    });
    worker.rpcMethods = {};
    function setup(ctx, rpcMethods, callbacks) {
        ctx.addEventListener('message', function (ref) {
            var data = ref.data;

            if (data.type === 'RPC') {
                var id = data.id;
                if (id != null) {
                    if (data.method) {
                        var method = rpcMethods[data.method];
                        if (method == null) {
                            ctx.postMessage({
                                type: 'RPC',
                                id: id,
                                error: 'NO_SUCH_METHOD'
                            });
                        } else {
                            Promise.resolve().then(function () { return method.apply(null, data.params); }).then(function (result) {
                                ctx.postMessage({
                                    type: 'RPC',
                                    id: id,
                                    result: result
                                });
                            }).catch(function (error) {
                                ctx.postMessage({
                                    type: 'RPC',
                                    id: id,
                                    error: error
                                });
                            });
                        }
                    } else {
                        var callback = callbacks[id];
                        if (callback == null) 
                            { throw Error(("Unknown callback " + id)); }
                        delete callbacks[id];
                        if (data.error) 
                            { callback.reject(Error(data.error)); }
                         else 
                            { callback.resolve(data.result); }
                    }
                }
            }
        });
    }
    
    setup(worker, worker.rpcMethods, callbacks);
    worker.call = (function (method, params) { return new Promise(function (resolve, reject) {
        var id = "rpc" + (++counter);
        callbacks[id] = {
            method: method,
            resolve: resolve,
            reject: reject
        };
        worker.postMessage({
            type: 'RPC',
            id: id,
            method: method,
            params: params
        });
    }); });
    var loop = function ( i ) {
        if (exports.hasOwnProperty(i) && !(i in worker)) {
            worker[i] = (function () {
                var args = [], len = arguments.length;
                while ( len-- ) args[ len ] = arguments[ len ];

                return worker.call(i, args);
            });
        }
    };

    for (var i in exports) loop( i );
    return worker;
}

function toCode(func) {
    return Function.prototype.toString.call(func);
}

function toCjs(code, exportsObjName, exports) {
    exportsObjName = exportsObjName || 'exports';
    exports = exports || {};
    code = code.replace(/^(\s*)export\s+default\s+/m, function (s, before) {
        exports.default = true;
        return ("" + before + exportsObjName + ".default = ");
    });
    code = code.replace(/^(\s*)export\s+(function|const|let|var)(\s+)([a-zA-Z$_][a-zA-Z0-9$_]*)/m, function (s, before, type, ws, name) {
        exports[name] = true;
        return ("" + before + exportsObjName + "." + name + " = " + type + ws + name);
    });
    return ("var " + exportsObjName + " = {};\n" + code + "\n" + exportsObjName + ";");
}

export default workerize;
//# sourceMappingURL=workerize.m.js.map
